% --------------------------------------------------------------------------

%  File name: N_eig.m

% -------------------------------------------------------------------------
%
%  This Matlab code solves the Lorentz eigenvalue complementarity problem:   
%     
%  x\in K, A*x - lamd*x \in K, <x, A*x - lamd*x>=0,  
%        
%  where A is a n X n dense matrix, and K = K1 X ... X Kr,
%  K1,..., Kr are second-order cones of dimensions n1,...,nr,
% 
%
%  This problem can be reformulated as the nonsmooth system of equations 
%                phiNR(xi,yi) = 0,   i=1,2,...,nq;
%                y = A*x - lamd*x - y=0; 
%              tr(xi1) + tr(xi2)+...+tr(xim)=1
%    




global A n nq nr nl;


nl = 0;

nq =10;                      % The number of second-order cones


Nint = 0;

flag = 0;

flag1 = 0;

Niter = 0;



lam = [];

vnum = [];

npro=1;

for npro = 1:5000
    
    [A,n,nr] = data(nq);
    
    [x0,y0,lamd0] = starting(n,nq,A);

%----------------------  Defining parameters  --------------------------

    tol = 10^(-8);
  
    maxiter =100;
  
    iter = 0;
  
%-----------------  The initialization part  --------------------------  
  
    x = x0;   y= y0;   lamd = lamd0;
 
%-----------------------------------   starting  ----------------------- 
  
    time0 = clock ;
    
    [phixy,trx] = profun(x,y);

    phi = [phixy; A*x-lamd*x-y; trx-1]; 


    %fprintf('\n No.| iter  norm(phi) lamd \n')
 
   % fprintf('%6d|  %6d  %6.4e  %10.8f \n', [Num,iter, norm(phi), lamd])
  

    while  norm(phi) > tol & iter < maxiter
     
         iter = iter  + 1;
   
         [phixy,trx,e,Vx,Vy] = profun(x,y);
   
         D = [Vx  Vy  zeros(n,1); A-lamd*eye(n)  -eye(n)  -x; e zeros(1,n) 0];
   
         D = D + 1.0e-6*eye(2*n+1);
         
         dir = -D\phi;

         x = x + dir(1:n);
   
         y = y + dir(n+1:2*n);
   
         lamd = lamd + dir(2*n+1);
   
         [phixy,trx] = profun(x,y);
   
         phi = [phixy; A*x-lamd*x-y;trx-1]; 
   
  end
       
  if  iter< maxiter
    
      flag = flag +1;
      
      Niter = Niter + iter;
     
   
     % fprintf('%6d| %6d %6.4e %10.8f \n', [Num,iter, norm(phi),lamd]) ;   
      
     if  npro==1
   
         lam = [lam;lamd];
         
         vnum = [vnum;1];
         
         if abs(x(1)-norm(x(2:n)))<=1.0e-8
             
             Nint = Nint + 1;
                
         end    
         
     else 
        
         vec = abs(lam-lamd);
   
         if all(vec>1.0e-5)
            
            lam = [lam;lamd];
            
            vnum = [vnum;1];          
            
        else
            
            index = find(vec<=1.0e-5); 
            
            vnum(index) = vnum(index)+1;
            
        end   
        
        if abs(x(1)-norm(x(2:n)))<=1.0e-8
             
                 Nint = Nint + 1;
                                     
        end  
        
     end   
 end
    
 npro = npro+1;

end

flag

Niter/flag

size(lam)

vnum

Nint