%----------------------------------------------------------
% File name:  phifb
%----------------------------------------------------------

% The code is used to evaluate the vector-function phifb(x,y) 
% and its B-subdifferential [Ux  Uy] at (x,y)
% w = x^2 + y^2,  sqrxy = w^{1/2}


function [f,trx,Ux,Uy] = phifb(x,y)

global m n nl nq nr null;


%-----------------------------  The initialization part  ----------------------
Ux = sparse(zeros(n));

Uy = Ux;

trx = 0;
w =[];
sqrxy = [];
%------- the FB function associated with K^1 ------
if nl > 0 
   xla = x(1:nl);
   yla = y(1:nl);
   w = xla.^2 + yla.^2;
   sqrxy = w.^(1/2);
   
   if nargout >2 
        for i = 1:nl 
            if ((xla(i)==0)&(yla(i)==0))
                 Ux(i,i) = 0;
                 Uy(i,i) = 1;
            else
                 Ux(i,i) = xla(i)/sqrxy(i);
                 Uy(i,i) = yla(i)/sqrxy(i);
            end
  
        end
   end 
   trx = sum(xla);
end

%--------- the FB function associated with K^q  -------

for i = 1 : nq
    ni1 = nr(i);
    ni2 = nr(i+1)-1;
    ni = ni2-ni1+1;
    
    xi = x(ni1:ni2);
    yi = y(ni1:ni2);
    
    xi1 = xi(1);   xi2 = xi(2:end);
    yi1 = yi(1);   yi2 = yi(2:end);

% ------------  compute wi = (wi1,wi2)  =  xi^2 + yi^2  ---------------------

    wi1 = xi'*xi + yi'*yi ;
    wi2 = 2*(xi1*xi2+yi1*yi2);
    wi = [wi1;wi2];
    nwi = norm(wi2);

%------------- compute sqrxy(ni1:ni2) = (xi^2 + yi^2 )^(1/2)  --------------- 
    

    lamd1 = sqrt( wi1 - nwi );

    lamd2 = sqrt( wi1 + nwi);
    
   if nwi > null
       
       sqrxyi = 0.5*[lamd1+lamd2; (lamd2 - lamd1)*(wi2/nwi)];
       
   else
       
       sqrxyi = lamd1*[1; zeros(ni-1,1)];
       
   end
      w = [w;wi];
      
       sqrxy = [sqrxy;sqrxyi];
    
   if nargout >2 
        
      if ( wi1 > nwi )              % the case that phiFB(z) is differentiable at the point z      

           Vx = Lx_inverse(sqrxyi)* Lx(xi);
           Vy = Lx_inverse(sqrxyi)* Lx(yi);
          
      else                           % the case that phiFB(z) is not differentiable at the point z    
          
          if  nwi>null
          
                 
              ui = (1/sqrt(2))*[1; -wi2/nwi];
              vi = (1/sqrt(2))*[1; -wi2/nwi];
                        
              Vx = (0.5/sqrt(xi1^2+yi1^2))*[xi1 xi2'; xi2  2*xi1*eye(ni-1)-(wi2/nwi)*xi2'] + 0.5*[1;-wi2/nwi]*ui';
             
              Vy = (0.5/sqrt(xi1^2+yi1^2))*[yi1 yi2'; yi2  2*yi1*eye(ni-1)-(wi2/nwi)*yi2'] + 0.5*[1;-wi2/nwi]*vi';
             
         else                      % the case that (x,y) = (0,0)
             gi = rand(ni2 - ni1 + 1,1);
             hi = [gi(1);-gi(2:ni2 - ni1 + 1)];
             ghi = sqrt(gi'*gi + hi'*hi);
             gi = gi/ghi;
             hi = hi/ghi;
             wi2 = rand(ni-1,1);
             bwi = wi2/norm(wi2);
             pi = (1/sqrt(2))*[1;bwi];
             qi = (1/sqrt(2))*[1;bwi];
              ui = [1;-bwi];
              vi = zeros(ni2-ni1+1,1);
             si =  (1/4)*(ui+pi);
             omgi = (3/4)*(vi+qi);
             Ax22 = (1/2)*[1;bwi]*pi' + (1/2)*[1;-bwi]*ui' + [0 si(2:end)'*(eye(ni-1)-bwi*bwi'); zeros(ni-1,1)  si(1)*(eye(ni-1)-bwi*bwi')]';
             Bx22 = (1/2)*[1;bwi]*pi' + (1/2)*[1;-bwi]*ui' + [0 omgi(2:end)'*(eye(ni-1)-bwi*bwi'); zeros(ni-1,1) omgi(1)*(eye(ni-1)-bwi*bwi')]';
             Vx =(1/2)*( Lx(gi) + Ax22 );
             Vy =(1/2)*( Lx(hi) + Bx22 );
         end
   end
   
   Ux(ni1:ni2,ni1:ni2) = sparse(Vx);
   Uy(ni1:ni2,ni1:ni2) = sparse(Vy);
   trx = trx + xi1;
   end
end
f = (x+y) - sqrxy;
