%--------------------------------------------------------------------------

%  File name: s_percent.m

% -------------------------------------------------------------------------
%
%  This Matlab code solves the Lorentz eigenvalue complementarity problem:   
%     
%  x\in K, A*x - lamd*x \in K, <x, A*x - lamd*x>=0,  
%        
%  where A is a n X n dense matrix, and K = K1 X ... X Kr,
%  K1,..., Kr are second-order cones of dimensions n1,...,nr,
%  and records the successful percentage of the semismooth method
%
% 
%  This problem can be reformulated as the system of normal equations 
%               F(Pz) + z - Pz = 0,   
%              tr(xi1) + tr(xi2)+...+tr(xim)=1
%    
%  where Pz means the projection of the vector z onto the cone K
%  the mapping F from R^n to R^n is defined as follows:
%  F(u) = Au - lamd u 





global A n nq nr nl;


nl = 0;

nq = 1

flag = 0;

Niter = 0;

npro=1;

lam=[];

for npro = 1:500
   
   [A,n,nr] = data(nq); 
    
   [x0,lamd0] = starting(n,nq,A);

%----------------------  Defining parameters  --------------------------

    tol = 10^(-8);    maxiter =300;    iter = 0;
  
    
%-----------------  The initialization part  --------------------------  

    x = x0;     lamd = lamd0;
 
   
%-----------------------------------   starting  ----------------------- 
  
   time0 = clock ;
  
    [xp,trx] = profun(x);

   phi = [A*xp-(lamd+1)*xp+x; trx-1]; 

    while norm(phi) > tol & iter < maxiter
     
          iter = iter  + 1;
   
        
          [xp,trx,e,Vx] = profun(x);
         
   
          D = [A*Vx-(lamd+1)*Vx+eye(n)  -xp; e  0];
   
          D = D + 1.0e-6*eye(n+1);
      
      
          dir = -D\phi;
          
          x = x + dir(1:n);
   
         lamd = lamd + dir(n+1:n+1);
   
         [xp,trx] = profun(x);

         phi = [A*xp-(lamd+1)*xp+x; trx-1]; 
  
       end
 
      if  iter< maxiter
          
          Niter = Niter + iter;
          
          flag = flag +1;
          
          lam =[lam;lamd];
      end
         
   
    if mod(npro,100)==0
        npro = npro +1
    else
        npro = npro +1;
    end
  end
flag/500
Niter/flag
  